var ExtractionPrep = function()
{
	var exprep = {};
	var debug = 0;
	
	exprep.REAGENTS = [
		'xyleneSpecimen', 'etoh995Specimen', 'bufferPkdSpecimen', 'bufferAtlSpecimen',
		'proteinaseKSpecimen', 'rnaseASpecimen', 'allPrepFfpeKit', 'etoh995',
		'bufferRlt', 'bufferFrn', 'bufferRpe', 'bufferAl',
		'bufferAw1', 'bufferAw2', 'bufferAte', 'bufferRdd',
		'rnaseFreeWater', 'dnaseMix', 'rneasyMinEluteSpinColumn', 'qiaampMinEluteSpinColumn'
		];

	var DEFAULT_RNA_VOLUME = 25;
	var DEFAULT_DNA_VOLUME = 25;
	var MINIMAL_PRE_NORMALIZE_QUANTITY = 1.1; // µg
	var EXTRA_QIACUBE_SAMPLE_NAME_START = 'E';
	var protocolMap = {};

	var sourceItemListIsValid = false;
	var numExtraQiacubeItemsIsValid = false;
	var extraItemsList = [];
	var qiacubeSortedPrefixes = 'CABDEFGHIJKLMNOPQRSTUVWXYZ';
	var qiacubeDateIsAfterIsolationDate = true;
	var qiacubeRunNoIsValid = false;
	var qiacubePositionsAreValid = false;
	var counterBalanceInfoRow = false;
	var nextLibraryPlateName = '';
	var trackingSheetPreview = true;
	var createdDnaPlate = [];
	
	// Page initialization
	exprep.initPage = function()
	{
		// Step 1
		Events.addEventHandler('step-1', 'wizard-validate', exprep.validateStep1);
		Events.addEventHandler('extractSourceItems', 'change', exprep.sourceItemsOnChange);
		Events.addEventHandler('numExtraQiacubeItems', 'change', exprep.numExtraQiacubeItemsOnChange);

		// Step 2
		Events.addEventHandler('step-2', 'wizard-initialize', exprep.initializeStep2);
		Events.addEventHandler('step-2', 'wizard-validate', exprep.validateStep2);
		
		Events.addEventHandler('isolationDate', 'change', Wizard.validateDate);
		Events.addEventHandler('qiacubeDate', 'change', Wizard.validateDate);
		Events.addEventHandler('isolationDate', 'change', exprep.qiacubeDateOnChange);
		Events.addEventHandler('qiacubeDate', 'change', exprep.qiacubeDateOnChange);
		Events.addEventHandler('qiacubeRunNo', 'change', exprep.qiacubeRunNoOnChange);
		
		// Step 3
		Events.addEventHandler('step-3', 'wizard-initialize', exprep.initializeStep3);
		Events.addEventHandler('step-3', 'wizard-validate', exprep.validateStep3);
	
		Buttons.addClickHandler('downloadSampleIdFile', exprep.downloadSampleIdFile);
		Buttons.addClickHandler('downloadQPcrSampleIdFile', exprep.downloadQPcrSampleIdFile);
		
		// Navigation
		Buttons.addClickHandler('gocancel', Wizard.cancelWizard);
		Buttons.addClickHandler('gorestart', Wizard.restartWizard);
		Buttons.addClickHandler('gonext', Wizard.goNextOnClick);
		Buttons.addClickHandler('goregister', Wizard.goRegister);
		Buttons.addClickHandler('gocreate', exprep.createProtocol);

		// Final registration
		Events.addEventHandler('wizard', 'wizard-submit', exprep.submit);

		var url = '../Extraction.servlet?ID='+App.getSessionId();
		url += '&cmd=GetUnprocessedItems';		
		Wizard.showLoadingAnimation('Loading unprocessed items...');
		Wizard.asyncJsonRequest(url, exprep.initializeStep1);
	}
	
	exprep.initializeStep1 = function(response)
	{
		var frm = document.forms['meludi'];
		frm.numExtraQiacubeItems.value = 0;
		extraItemsList = [];
		var specimens = response.specimens;
		var rnas = response.rnas;
		var dnas = response.dnas;
		var items = response.items;
		
		var sourceItemList = frm.extractSourceItems;
		if (items.length)
		{
			var numItems = items.length;
			for (var i=0; i < items.length; i++)
			{
				var item = items[i];
				var name = (i+1) + ': ' + Strings.encodeTags(item.name);
				if (item.bioWell)
				{
					name += ' -- ' + Strings.encodeTags(item.bioWell.bioPlate.name + ' (' + item.bioWell.location + ')');
				}
				var selected = i < 24;
				// Identification of item via id works here as specimens and extracts
				// are stored in the same database table using discriminators, and are
				// therefore guaranteed to not have the same id value as another item in the table.
				var option = new Option(name, item.id, selected, selected);
				option.item = item;
				sourceItemList.options[sourceItemList.length] = option;
			}
			exprep.sourceItemsOnChange();
		}
		else
		{
			Wizard.setFatalError('No items available for processing.');
			return;
		}

		Doc.show('step-1');
		Doc.show('gonext');
	}

	exprep.sourceItemsOnChange = function()
	{
		sourceItemListIsValid = false;
		Wizard.setInputStatus('extractSourceItems');
		
		var frm = document.forms['meludi'];
		var sourceItemList = frm.extractSourceItems;
		
		var numSelected = 0;
		for (var i = 0; i < sourceItemList.length; i++)
		{
			if (sourceItemList[i].selected) numSelected++;
		}

		if (numSelected < 1 || numSelected > 24)
		{
			Wizard.setInputStatus('extractSourceItems', 'invalid', 'Must select 1-24 items.');
			return;
		}
		
		sourceItemListIsValid = true;
		Wizard.setInputStatus('extractSourceItems', 'valid');

		exprep.numExtraQiacubeItemsOnChange();
	}

	
	exprep.numExtraQiacubeItemsOnChange = function()
	{
		var frm = document.forms['meludi'];
		var numExtraQiacubeItemsStr = frm.numExtraQiacubeItems.value;
		numExtraQiacubeItemsIsValid = false;

		if (numExtraQiacubeItemsStr == '')
		{
			Wizard.setInputStatus('numExtraQiacubeItems', 'invalid', 'Missing');
			return;
		}
		var numExtraQiacubeItems = parseInt(numExtraQiacubeItemsStr, 10);
		if (numExtraQiacubeItems < 0)
		{
			Wizard.setInputStatus('numExtraQiacubeItems', 'invalid', 'Must be at least 0');
			return;		
		}

		var sourceItemList = frm.extractSourceItems;
		var numSelected = 0;
		for (var i = 0; i < sourceItemList.length; i++)
		{
			if (sourceItemList[i].selected) numSelected++;
		}
		var maxAvailable = 24 - numSelected;
		if (numExtraQiacubeItems > maxAvailable)
		{
			Wizard.setInputStatus('numExtraQiacubeItems', 'invalid', 'Only ' + maxAvailable + ' spots available');
			return;		
		}
		
		Wizard.setInputStatus('numExtraQiacubeItems', 'valid');
		numExtraQiacubeItemsIsValid = true;
	}

	
	exprep.validateStep1 = function(event)
	{
		if (!sourceItemListIsValid || !numExtraQiacubeItemsIsValid)
		{
			event.preventDefault();
		}
	}
	
	exprep.initializeStep2 = function()
	{
		var frm = document.forms['meludi'];
		frm.extractSourceItems.disabled = true;
/*
		frm.rnaDefaultVolume.value = DEFAULT_RNA_VOLUME;
		frm.dnaDefaultVolume.value = DEFAULT_DNA_VOLUME;
*/
		
		Wizard.setCurrentStep(2);
		Doc.show('gocancel');
		Doc.show('gonext');

		Meludi.loadProtocols('SAMPLE_HANDLING_PROTOCOL', 'extractionProtocol');
		Doc.addClass('extractionProtocol', 'list-loading');
		frm.extractionProtocol[0] = new Option('loading...', '');
		var url = '../Protocol.servlet?ID='+App.getSessionId();
		url += '&cmd=GetProtocols&subtype=EXTRACTION_PROTOCOL';
		Wizard.asyncJsonRequest(url, exprep.protocolsLoaded);

		frm.isolationDate.focus();
		Events.sendChangeEvent('isolationDate');
		Events.sendChangeEvent('qiacubeDate');
	}
	
	exprep.protocolsLoaded = function(response)
	{
		var frm = document.forms['meludi'];
		var protocols = response.protocols;

		Doc.removeClass('extractionProtocol', 'list-loading');
		frm.extractionProtocol.length = 0;
		for (var i = 0; i < protocols.length; i++)
		{
			var name = protocols[i].name;
			var id = protocols[i].id;
			protocolMap[id] = name;
			frm.extractionProtocol[frm.extractionProtocol.length] = new Option(name, id, false, true);
		}		
		frm.extractionProtocol[frm.extractionProtocol.length] = new Option('- none -', '');

		var url = '../Extraction.servlet?ID='+App.getSessionId();
		url += '&cmd=GetLastProcessedDna';
		Wizard.asyncJsonRequest(url, exprep.lastProcessedDnaLoaded);
	}
	
	exprep.lastProcessedDnaLoaded = function(response)
	{
		var frm = document.forms['meludi'];
		var dnas = response.dnas;

		if (dnas.length > 0)
		{
			if (dnas.length > 0)
			{
				var dna = dnas[0];

				// Reset the existing values

				// AllPrep FFPE Kit			
				var allPrepFfpeKit = '';

/*
				// Specimen reagents
				var specXylene = '';
				var specEtoh995 = '';
				var specBufferPkd = '';
				var specBufferAtl = '';
				var specProteinaseK = '';
				var specRNAseA = '';

				// QIAcube reagents
				var bufferRlt = '';
				var etoh995 = '';
				var bufferFrn = '';
				var bufferRpe = '';
				var rnaseFreeWater = '';
				var dnaseMix = '';
				var bufferRdd = '';
				var rneasyMinEluteSpinColumn = '';
				var bufferAl = '';
				var bufferAW1 = '';
				var bufferAW2 = '';
				var bufferAte = '';
				var qiaampMinEluteSpinColumn = '';
*/

				if (dna) 
				{
					// AllPrep FFPE Kit			
					allPrepFfpeKit = dna.allPrepFfpeKit;

/*
					// Specimen reagents
					specXylene = dna.xyleneSpecimen;
					specEtoh995 = dna.etoh995Specimen;
					specBufferPkd = dna.bufferPkdSpecimen;
					specBufferAtl = dna.bufferAtlSpecimen;
					specProteinaseK = dna.proteinaseKSpecimen;
					specRNAseA = dna.rnaseASpecimen;

					// QIAcube reagents
					bufferRlt = dna.bufferRlt;
					etoh995 = dna.etoh995;
					bufferFrn = dna.bufferFrn;
					bufferRpe = dna.bufferRpe;
					rnaseFreeWater = dna.rnaseFreeWater;
					dnaseMix = dna.dnaseMix;
					bufferRdd = dna.bufferRdd;
					rneasyMinEluteSpinColumn = dna.rneasyMinEluteSpinColumn;
					bufferAl = dna.bufferAl;
					bufferAW1 = dna.bufferAw1;
					bufferAW2 = dna.bufferAw2;
					bufferAte = dna.bufferAte;
					qiaampMinEluteSpinColumn = dna.qiaampMinEluteSpinColum;
*/
				}

				// Update reagent values

				// AllPrep FFPE Kit			
				frm.allPrepFfpeKit.value = allPrepFfpeKit;

/*
				// Specimen reagents
				frm.xyleneSpecimen.value = specXylene;
				frm.etoh995Specimen.value = specEtoh995;
				frm.bufferPkdSpecimen.value = specBufferPkd;
				frm.bufferAtlSpecimen.value = specBufferAtl;
				frm.proteinaseKSpecimen.value = specProteinaseK;
				frm.rnaseASpecimen.value = specRNAseA;

				// QIAcube reagents
				frm.etoh995.value = etoh995;
				frm.bufferRlt.value = bufferRlt;
				frm.bufferFrn.value = bufferFrn;
				frm.bufferRpe.value = bufferRpe;
				frm.bufferAl.value = bufferAl;
				frm.bufferAw1.value = bufferAW1;
				frm.bufferAw2.value = bufferAW2;
				frm.bufferAte.value = bufferAte;
				frm.bufferRdd.value = bufferRdd;
				frm.rnaseFreeWater.value = rnaseFreeWater;
				frm.dnaseMix.value = dnaseMix;
				frm.rneasyMinEluteSpinColumn.value = rneasyMinEluteSpinColumn;
				frm.qiaampMinEluteSpinColumn.value = qiaampMinEluteSpinColumn;
*/
			}
		}

		var url = '../Extraction.servlet?ID='+App.getSessionId();
		url += '&cmd=GetNextStartPlateName';
		Wizard.asyncJsonRequest(url, exprep.startPlateNameLoaded);
	}
	
	exprep.startPlateNameLoaded = function(response)
	{
		var frm = document.forms['meludi'];
		var nextStartPlateName = response.nextStartPlateName;
		//alert("exprep.startPlateNameLoaded(): nextStartPlateName = " + nextStartPlateName);
		frm.startPlateName.value = nextStartPlateName;
	}
	
	exprep.qiacubeDateOnChange = function()
	{
		var frm = document.forms['meludi'];
		
		var isolationDateIsValid = Wizard.isValid('isolationDate');
		var qiacubeDateIsValid = Wizard.isValid('qiacubeDate');
		
		if (!isolationDateIsValid || !qiacubeDateIsValid) return;

		var isolationDate = Dates.parseString(frm.isolationDate.value, 'yyyyMMdd');
		var qiacubeDate = Dates.parseString(frm.qiacubeDate.value, 'yyyyMMdd');
		if (isolationDate > qiacubeDate)
		{
			Wizard.setInputStatus('qiacubeDate', 'invalid', 'QIAcube date is before isolation date');
			qiacubeDateIsAfterIsolationDate = false;
			return;
		}

		qiacubeDateIsAfterIsolationDate = true;

		Wizard.setInputStatus('qiacubeDate', 'valid');
		var url = '../Extraction.servlet?ID='+App.getSessionId();
		url += '&cmd=GetNextQiacubeRunNo&qiacubeDate='+encodeURIComponent(frm.qiacubeDate.value);
		Wizard.showLoadingAnimation('Loading Qiacube run number...');
		Wizard.asyncJsonRequest(url, exprep.gotNextRunNumber);
	}

	exprep.gotNextRunNumber = function(response)
	{
		var frm = document.forms['meludi'];
		frm.qiacubeRunNo.value = response.nextQiacubeRunNo;
		Events.sendChangeEvent('qiacubeRunNo');
	}

	exprep.qiacubeRunNoOnChange = function()
	{
		var frm = document.forms['meludi'];
		var runNumber = frm.qiacubeRunNo.value;
		qiacubeRunNoIsValid = false;

		if (runNumber == '')
		{
			Wizard.setInputStatus('qiacubeRunNo', 'invalid', 'Missing');
			return;
		}
		if (!(parseInt(runNumber, 10) >= 1))
		{
			Wizard.setInputStatus('qiacubeRunNo', 'invalid', 'Must be at least 1');
			return;		
		}
		
		Wizard.setInputStatus('qiacubeRunNo', 'valid');
		qiacubeRunNoIsValid = true;
	}

	
	exprep.validateStep2 = function(event)
	{
		var valid = true;
		valid &= Wizard.isValid('isolationDate');
		valid &= Wizard.isValid('qiacubeDate');
		valid &= qiacubeRunNoIsValid;
		
		if (!valid) event.preventDefault();
	}

	
	exprep.initializeStep3 = function()
	{
		var frm = document.forms['meludi'];
		var startPlateName = frm.startPlateName.value;
		Doc.element('detailsStartPlateName').innerHTML = '<b>Start plate: ' + startPlateName + '</b>';
/*
		var rnaDefaultVolume = exprep.getNumber(frm.rnaDefaultVolume.value);
		var dnaDefaultVolume = exprep.getNumber(frm.dnaDefaultVolume.value);
*/
		qiacubeSortedPrefixes = exprep.getQiacubeSortedPrefixes();
		
		var html = '<table id="detailsTable" class="step-form">';
		// First header row
		html += '<thead>';
		html += '<tr>';
		html += '<th>Extract</th>';
		html += '<th class="dottedleft">QIAcube</th>';
		html += '<th colspan="1" class="dottedleft"></th>';
		html += '<th colspan="1" class="dottedleft">NanoDrop</th>';
/*
		html += '<th colspan="5" class="dottedleft">RNA</th>';
*/
/*
		html += '<th colspan="3" class="dottedleft">RNA</th>';
		html += '<th colspan="4" class="dottedleft">DNA</th>';
*/
		html += '<th class="dottedleft comment">Comment</th>';
		html += '<th></th>';
		html += '</tr>';
		// Second header row
		html += '<tr>';
		html += '<th>source item</th>';
		html += '<th class="dottedleft">position</th>';
		html += '<td class="dottedleft">Storage</td>';
/*
		html += '<td class="dottedleft">Pos</td><td>Vol.</td><td>Conc.</td><td>Quantity</td><td><span id="toggle-normalize-1" class="link" title="Toggle all">Pre-</span></td>';
*/
/*
		html += '<td class="dottedleft">Pos</td><td>Vol.</td><td>Conc.</td><td>Quantity</td><td>Pre-</td>';
*/
/*
		html += '<td class="dottedleft">Pos</td><td>Vol.</td><td>Conc.</td><td>Quantity</td>';
		html += '<td class="dottedleft">Vol.</td><td>Conc.</td><td>Quantity</td><td>ΔCt</td>';
*/
		html += '<th class="dottedleft">Plate Pos.</th>';
		html += '<td class="dottedleft comment"></td>';
		html += '<td></td>';
		html += '</tr>';
		// Third header row
		html += '<tr>';
		html += '<th></th>';
		html += '<th class="dottedleft"></th>';
		html += '<td class="dottedleft">box</td>';
/*
		html += '<td class="dottedleft"></td><td>(µl)</td><td>(ng/µl)</td><td>(µg)</td><td><span id="toggle-normalize-2" class="link" title="Toggle all">normalize</span></td>';
*/
/*
		html += '<td class="dottedleft"></td><td>(µl)</td><td>(ng/µl)</td><td>(µg)</td><td>normalize</td>';
*/
/*
		html += '<td class="dottedleft"></td><td>(µl)</td><td>(ng/µl)</td><td>(µg)</td>';
		html += '<td class="dottedleft">(µl)</td><td>(ng/µl)</td><td>(µg)</td><td></td>';
*/
		html += '<td class="dottedleft"></td>';
		html += '<td class="dottedleft comment"></td>';
		html += '<td></td>';
		html += '</tr>';
		html += '</thead>';
		html += '<tbody>';

		// Get list of selected extract source items
		var selItemsList = exprep.getSelectedItemsList();
		var numItems = selItemsList.length;
		var numSpecimenItems = exprep.getNumberOfSpecimenItems();
		var numRnaItems = exprep.getNumberOfRnaItems();
		var numDnaItems = exprep.getNumberOfDnaItems();
		var numNanoDropItems = numSpecimenItems + numRnaItems;
		// Get number of extra QIAcube items
		var numExtraQiacubeItems = 0;
		var numExtraQiacubeItemsStr = frm.numExtraQiacubeItems.value;
		if (numExtraQiacubeItemsStr && (numExtraQiacubeItemsStr != ''))
		{
			numExtraQiacubeItems = parseInt(numExtraQiacubeItemsStr, 10);
		}
		var numQiacubeItems = numSpecimenItems + numExtraQiacubeItems;
		var totNumItems = numItems + numExtraQiacubeItems;
		extraItemsList = exprep.getExtraItemsList(numExtraQiacubeItems);
		var itemNo = 0;
		var qiacubeItemNo = 0;
		// Set default values for DNA/RNA extraction
/*
		for (var i = 0; i < numItems; i++)
*/
		for (var i = 0; i < totNumItems; i++)
		{
			itemNo++;
			var item = null;
			var storageBox = '-';
			if (i < numItems)
			{
/*
				var item = selItemsList[i];
*/
				item = selItemsList[i];
				var qiacubePos = null;
				if (exprep.isSpecimen(item))
				{
					qiacubeItemNo++;
/*
					//qiacubePos = exprep.getDetailedQiaCubePosition(numSpecimenItems, qiacubeItemNo);
*/
					qiacubePos = exprep.getDetailedQiaCubePosition(numQiacubeItems, qiacubeItemNo);
				}
				item.qiacubePosition = qiacubePos;
				item.originalQiacubePosition = qiacubePos;
				
				var rna = {};
				var dna = {};
				// RNA
				rna.name = item.name + '.r';
/*
				rna.volume = rnaDefaultVolume;
*/
				// DNA
				dna.name = item.name + '.d';
/*
				dna.volume = dnaDefaultVolume;
*/
				
				item.rna = rna;
				item.dna = dna;
/*
				var storageBox = '-';
*/
				if (item.bioWell != null)
				{
					storageBox = Strings.encodeTags(item.bioWell.bioPlate.name + '[' + item.bioWell.location + ']');
				}
			}
			else
			{
				// Extra QIAcube item
				item = extraItemsList[i - numItems];
				qiacubeItemNo++;
				var qiacubePos = exprep.getDetailedQiaCubePosition(numQiacubeItems, qiacubeItemNo);
				item.qiacubePosition = qiacubePos;
				item.originalQiacubePosition = qiacubePos;
			}
				
			html += '<tr class="highlight">';
			html += '<th>' + Strings.encodeTags(item.name) + '</th>';
			html += '<td id="qiacubePosition'+i+'" class="dottedleft"></td>';
			html += '<td class="dottedleft">'+storageBox+'</td>';
			html += '<td id="rnaPos'+i+'" class="dottedleft"></td>';
/*
			html += '<td id="rnaVol'+i+'"></td>';
			html += '<td id="rnaConc'+i+'" class="bg-filled-50 italic"></td>';
			html += '<td id="rnaQuantity'+i+'" class="bg-filled-50 italic"></td>';
*/
/*
			html += '<td id="rnaPreNormalize'+i+'"name="rnaPreNormalize'+i+'"></td>';
*/
/*
			html += '<td id="dnaVol'+i+'"></td>';
			html += '<td id="dnaConc'+i+'">-</td>';
			html += '<td id="dnaQuantity'+i+'" class="bg-filled-50 italic"></td>';
			html += '<td id="deltaCt'+i+'" class="bg-filled-50 italic">-</td>';
*/
			html += '<td id="comments'+i+'" class="dottedleft comment"></td>';
			if (exprep.isSpecimen(item))
			{
				html += '<td><div class="button basicbutton interactable edit-details" style="width: 5em;" data-index="'+i+'">Edit&hellip;</div></td>';
			}
			else
			{
				html += '<td>-</td>';
			}
			//html += '<td><div class="button basicbutton interactable edit-details" style="width: 5em;" data-index="'+i+'">Edit&hellip;</div></td>';
			html += '</tr>';
		}
/*
		if (numSpecimenItems == 1 || numSpecimenItems == 11 || numSpecimenItems == 23)
*/
		if (numQiacubeItems == 1 || numQiacubeItems == 11 || numQiacubeItems == 23)
		{
			// Add extra row for counterbalance in QIAcube
			counterBalanceInfoRow = true;
			var i = numItems;
/*
			var qiacubePos = exprep.getDetailedQiaCubePosition(numSpecimenItems + 1, numSpecimenItems + 1);
*/
			var qiacubePos = exprep.getDetailedQiaCubePosition(numQiacubeItems + 1, numQiacubeItems + 1);
				
			html += '<tr class="highlight">';
			html += '<th>' + Strings.encodeTags('COUNTERBALANCE') + '</th>';
			html += '<td id="qiacubePosition'+i+'" class="dottedleft">'+qiacubePos+'</td>';
			html += '<td class="dottedleft">-</td>';
			html += '<td id="rnaPos'+i+'" class="dottedleft">-</td>';
/*
			html += '<td id="rnaVol'+i+'">-</td>';
			html += '<td id="rnaConc'+i+'" class="bg-filled-50 italic">-</td>';
			html += '<td id="rnaQuantity'+i+'" class="bg-filled-50 italic">-</td>';
*/
/*
			html += '<td>-</td>';
*/
/*
			html += '<td id="dnaVol'+i+'">-</td>';
			html += '<td id="dnaConc'+i+'">-</td>';
			html += '<td id="dnaQuantity'+i+'" class="bg-filled-50 italic">-</td>';
			html += '<td id="deltaCt'+i+'" class="bg-filled-50 italic">-</td>';
*/
			html += '<td id="comments'+i+'" class="dottedleft comment">Reminder to use counterbalance</td>';
			html += '<td>-</td>';
			html += '</tr>';				
		}
		html += '</tbody>';
		html += '</table>';
		Doc.element('detailsSection').innerHTML = html;
		var buttons = Doc.element('detailsSection').getElementsByClassName('edit-details');
		for (var i = 0; i < buttons.length; i++)
		{
			Buttons.addClickHandler(buttons[i], exprep.editDnaRnaDetails);
		}

		exprep.updateDetailsTable();
		
		Wizard.setCurrentStep(3);
		Doc.show('gocancel');
		Doc.show('goregister');
		Doc.show('gocreate');
		//Doc.hide('gocreate');

		Wizard.keepSessionAlive();
		
		if (debug && location.protocol != 'https:') 
		{
			Wizard.showDebugTools();
		}
	}

	exprep.getQiacubeSortedPrefixes = function()
	{
		var frm = document.forms['meludi'];
		var primaryQiacubePrefix = frm.qiacubePrimaryPrefix.value;
		var secondaryQiacubePrefix = frm.qiacubeSecondaryPrefix.value;

		// Get string of current QIAcube prefixes in order
		var fullSortedPrefixSet = 'ABCDEFGHIJKLMNOPQRSTUVWXYZ';
		var sortedPrefixes = '' + primaryQiacubePrefix + secondaryQiacubePrefix;
		for (var i = 0; i < fullSortedPrefixSet.length; i++)
		{
			// Get next prefix
			var prefix = fullSortedPrefixSet.substring(i, i+1);
			// Add prefix to string of sorted prefixes, if not already present
			if (prefix != primaryQiacubePrefix && prefix != secondaryQiacubePrefix)
			{
				sortedPrefixes += prefix;
			}
		}
		return sortedPrefixes;
	}

	exprep.getDetailedQiaCubePosition = function(nitems, i)
	{
		var dpos = null;
		var qiacubePos = null;
		// Two QIAcube instruments are available, denoted 'C' (default) and 'A'
		var qiacubeId = qiacubeSortedPrefixes.substring(0, 1);
		if (nitems == null || nitems < 1 || nitems > 24)
		{
			return null;
		}
		if (i == null || i < 1 || i > nitems)
		{
			return null;
		}
		if (nitems <= 12)
		{
			qiacubePos = exprep.getQiaCubePosition(nitems, i);
		}
		else
		{
			var numItemsQiaCube1 = Math.floor((nitems + 1)/2);
			if (numItemsQiaCube1 == 11)
			{
				numItemsQiaCube1 = 12;
			}
			if (i <= numItemsQiaCube1)
			{
				// Place sample in first QIAcube
				qiacubePos = exprep.getQiaCubePosition(numItemsQiaCube1, i);
			}
			else
			{
				// Place sample in second QIAcube
				var numItemsQiaCube2 = nitems - numItemsQiaCube1;
				qiacubePos = exprep.getQiaCubePosition(numItemsQiaCube2, i - numItemsQiaCube1);
				qiacubeId = qiacubeSortedPrefixes.substring(1, 2);
			}
		}
		if (qiacubeId != null && qiacubePos != null)
		{
			dpos = qiacubeId + qiacubePos;
		}
		return dpos;
	}

	exprep.getQiaCubePosition = function(nitems, i)
	{
		var pos = null;
		if (nitems == null || nitems < 1 || nitems > 12)
		{
			return null;
		}
		if (i == null || i < 1 || i > nitems)
		{
			return null;
		}
		var posArr = ['1','2','3','4','5','6','7','8','9','10','11','12'];
		if (nitems == 2)
		{
			posArr = ['1','7','','','','','','','','','',''];
		}
		else if (nitems == 3)
		{
			posArr = ['1','5','9','','','','','','','','',''];
		}
		else if (nitems == 4)
		{
			posArr = ['1','2','7','8','','','','','','','',''];
		}
		else if (nitems == 5)
		{
			posArr = ['1','2','5','8','9','','','','','','',''];
		}
		else if (nitems == 6)
		{
			posArr = ['1','2','3','7','8','9','','','','','',''];
		}
		else if (nitems == 7)
		{
			posArr = ['1','2','3','6','7','9','10','','','','',''];
		}
		else if (nitems == 8)
		{
			posArr = ['1','2','3','4','7','8','9','10','','','',''];
		}
		else if (nitems == 9)
		{
			posArr = ['1','2','3','5','6','7','9','10','11','','',''];
		}
		else if (nitems == 10)
		{
			posArr = ['1','2','3','4','5','7','8','9','10','11','',''];
		}
		else if (nitems == 11)
		{
			// Counterbalance blank vial filled with water needed i position 12.
			posArr = ['1','2','3','4','5','6','7','8','9','10','11',''];
		}
		pos = posArr[i-1];
		return pos;
	}

	exprep.getSelectedItemsList = function()
	{
		var frm = document.forms['meludi'];
		
		// Get number of selected extract source items
		var numItems = 0;
		var selItemsList = [];
		for (var i = 0; i < frm.extractSourceItems.length; i++)
		{
			if (frm.extractSourceItems[i].selected)
			{
				if (frm.extractSourceItems[i].item != null)
				{
					selItemsList[numItems] = frm.extractSourceItems[i].item;
					numItems++;
				}
			}
		}
		return selItemsList;
	}

	exprep.getExtraItemsList = function(numExtraItems)
	{
		// Create extra items
		for (var i = 0; i < numExtraItems; i++)
		{
			var no = i + 1;
			var numStr = '' + no;
			if (no < 10)
			{
				numStr = '0' + numStr;
			}
			var jsonString = '{"name":"' + EXTRA_QIACUBE_SAMPLE_NAME_START + numStr + '","qiacubePosition":"","originalQiacubePosition":""}';
			var extraItem = JSON.parse(jsonString);
			extraItemsList[i] = extraItem;
		}
		return extraItemsList;
	}

	exprep.isSpecimen = function(item)
	{
		var isSpecimen = false;
		if (item != null)
		{
			if (!exprep.isRna(item) && !exprep.isDna(item))
			{
				var name = item.name;
				if (name != null)
				{
					// Check if item name does not start with string EXTRA_QIACUBE_SAMPLE_NAME_START
					if (name.indexOf(EXTRA_QIACUBE_SAMPLE_NAME_START) != 0)
					{
						isSpecimen = true;
					}
				}
			}
		}
		return isSpecimen;
	}

	exprep.isRna = function(item)
	{
		var isRna = false;
		if (item != null)
		{
			var name = item.name;
			if (name != null)
			{
				// Check if item name ends with ".r"
				if (name.indexOf('.r') == (name.length - 2))
				{
					isRna = true;
				}
			}
		}
		return isRna;
	}

	exprep.isDna = function(item)
	{
		var isDna = false;
		if (item != null)
		{
			var name = item.name;
			if (name != null)
			{
				// Check if item name ends with ".d"
				if (name.indexOf('.d') == (name.length - 2))
				{
					isDna = true;
				}
			}
		}
		return isDna;
	}

	exprep.getNumberOfSpecimenItems = function()
	{
		// Get list of selected extract source items
		var selItemsList = exprep.getSelectedItemsList();
		var numItems = selItemsList.length;
		var numSpecimen = 0;
		for (var i = 0; i < numItems; i++)
		{
			var item = selItemsList[i];
			if (exprep.isSpecimen(item))
			{
				numSpecimen++;
			}
		}
		return numSpecimen;
	}
	
	exprep.getNumberOfRnaItems = function()
	{
		// Get list of selected extract source items
		var selItemsList = exprep.getSelectedItemsList();
		var numItems = selItemsList.length;
		var numRna = 0;
		for (var i = 0; i < numItems; i++)
		{
			var item = selItemsList[i];
			if (exprep.isRna(item))
			{
				numRna++;
			}
		}
		return numRna;
	}
	
	exprep.getNumberOfDnaItems = function()
	{
		// Get list of selected extract source items
		var selItemsList = exprep.getSelectedItemsList();
		var numItems = selItemsList.length;
		var numDna = 0;
		for (var i = 0; i < numItems; i++)
		{
			var item = selItemsList[i];
			if (exprep.isDna(item))
			{
				numDna++;
			}
		}
		return numDna;
	}

	exprep.updateDetailsTable = function()
	{
		var frm = document.forms['meludi'];
/*
		var rnaDefaultVolume = exprep.getNumber(frm.rnaDefaultVolume.value);
		var dnaDefaultVolume = exprep.getNumber(frm.dnaDefaultVolume.value);
*/
		var hasQiacubePosition = [];
		var duplicatePosition = 0;

		// Get list of selected extract source items
		var selItemsList = exprep.getSelectedItemsList();
		var numItems = selItemsList.length;
		// Get number of extra QIAcube items
		var numExtraQiacubeItems = 0;
		var numExtraQiacubeItemsStr = frm.numExtraQiacubeItems.value;
		if (numExtraQiacubeItemsStr && (numExtraQiacubeItemsStr != ''))
		{
			numExtraQiacubeItems = parseInt(numExtraQiacubeItemsStr, 10);
		}
		var totNumItems = numItems + numExtraQiacubeItems;
		//var extraItemsList = exprep.getExtraItemsList(numExtraQiacubeItems);
		// Get sorted list of used integer QIAcube positions for specimens, generalized for more than one QIAcube
		var specSortedIntQiacubePosition = [];
/*
		var specItemNo = 0;
		for (var i = 0; i < numItems; i++)
*/
		var qiacubeItemNo = 0;
		for (var i = 0; i < totNumItems; i++)
		{
			if (i < numItems)
			{
				var item = selItemsList[i];
				if (exprep.isSpecimen(item))
				{
					var intQiacubePosition = exprep.getIntQiacubePosition(item.qiacubePosition);
					specSortedIntQiacubePosition[qiacubeItemNo] = intQiacubePosition;
					qiacubeItemNo++;
				}
			}
			else
			{
				// Extra QIAcube item
				var item = extraItemsList[i - numItems];
				var intQiacubePosition = exprep.getIntQiacubePosition(item.qiacubePosition);
				specSortedIntQiacubePosition[qiacubeItemNo] = intQiacubePosition;
				qiacubeItemNo++;
			}
		}
		// Sort for ascending integer values
		specSortedIntQiacubePosition.sort(function(a,b){return a-b});
		var numSpecimenItems = exprep.getNumberOfSpecimenItems();
		var numRnaItems = exprep.getNumberOfRnaItems();
		var numDnaItems = exprep.getNumberOfDnaItems();
		var numNanoDropItems = numSpecimenItems + numRnaItems;
		var rnaItemNo = 0;
		// Set default values for DNA/RNA extraction
/*
		for (var i = 0; i < numItems; i++)
*/
		for (var i = 0; i < totNumItems; i++)
		{
/*
			var item = selItemsList[i];
*/
			var item = null;
			if (i < numItems)
			{
				item = selItemsList[i];
			}
			else
			{
				// Extra QIAcube item
				item = extraItemsList[i - numItems];
			}
			Doc.element('qiacubePosition'+i).innerHTML = item.qiacubePosition;
			Doc.addOrRemoveClass('qiacubePosition'+i, 'nondefault', item.qiacubePosition != item.originalQiacubePosition);

			var intQiacubePosition = exprep.getIntQiacubePosition(item.qiacubePosition);
			if (intQiacubePosition != null)
			{
				if (hasQiacubePosition[intQiacubePosition])
				{
					duplicatePosition = intQiacubePosition;
				}
				else
				{
					hasQiacubePosition[intQiacubePosition] = true;
				}
			}
			var intNanoDropPlatePosition = intQiacubePosition;
			// Get integer NanoDrop position for specimens from index in sorted list of used integer QIAcube positions
			if (intQiacubePosition)
			{
				var sortIndex = -1;
				for (var j = 0; j < specSortedIntQiacubePosition.length; j++)
				{
					if (intQiacubePosition == specSortedIntQiacubePosition[j])
					{
						sortIndex = j;
					}
				}
				if (sortIndex > -1)
				{
					intNanoDropPlatePosition = sortIndex + 1;
				}
			}
			if (exprep.isRna(item))
			{
				rnaItemNo++;
				intNanoDropPlatePosition = numSpecimenItems + rnaItemNo;
			}
			
			// Calculate NanoDrop well from Qiacube position
			var row;
			var col;
			// Specimens from up to two QIAcube instruments and input RNA items placed on one NanoDrop plate
			if (intNanoDropPlatePosition > 16)
			{
				// (17 - 24) NanoDrop plate for QIAcube 'A', positions A3 - H3
				row = intNanoDropPlatePosition - 17;
				col = 3;
			}
			else if (intNanoDropPlatePosition > 8)
			{
				// ( 9 - 12) NanoDrop plate for QIAcube 'C', positions A2 - D2
				// (13 - 16) NanoDrop plate for QIAcube 'A', positions E2 - H2
				row = intNanoDropPlatePosition - 9;
				col = 2;
			}
			else if (intNanoDropPlatePosition > 0)
			{
				// ( 1 -  8) NanoDrop plate for QIAcube 'C', positions A1 - H1
				row = intNanoDropPlatePosition - 1;
				col = 1;
			}
			if (i < numItems)
			{
				if (!exprep.isDna(item))
				{
/*
					item.dna.well = Meludi.wellToAlpha(row) + (col+3);
*/
					item.rna.well = Meludi.wellToAlpha(row) + col;
				}
				else
				{
					item.dna.well = null;
					item.rna.well = null;
				}

/*
				Doc.element('dnaVol'+i).innerHTML = item.dna.volume;
				Doc.addOrRemoveClass('dnaVol'+i, 'nondefault', item.dna.volume != dnaDefaultVolume);
*/
/*
				var dnaConc = Doc.element('dnaConc'+i);
				dnaConc.value = Meludi.formatNumber(item.dna.ndConc) || '';
				dnaConc.title = '260/280=' + item.dna.nd260by280 + '\n260/230=' + item.dna.nd260by230;
				if (dnaConc != null)
				{
					var dnaConcValue = dnaConc.value;
					if (dnaConcValue != null && dnaConcValue != '-' && dnaConcValue != '')
					{
						item.dna.ndConc = exprep.getNumber(dnaConcValue);
					}
				}
				item.dna.quantity = item.dna.qubitConc * item.dna.volume / 1000;
				Doc.element('dnaQuantity'+i).innerHTML = Meludi.formatNumber(item.dna.quantity || null, null, 2) || '-';

				var deltaCt = Doc.element('deltaCt'+i);
				if (deltaCt != null)
				{
					deltaCt.innerHTML = Meludi.formatNumber(item.dna.deltaCt) || '-';
				}
*/			
				Doc.element('rnaPos'+i).innerHTML = item.rna.well;
				Doc.addOrRemoveClass('rnaPos'+i, 'nondefault', item.qiacubePosition != item.originalQiacubePosition);
/*
				Doc.element('rnaVol'+i).innerHTML = item.rna.volume;
				Doc.addOrRemoveClass('rnaVol'+i, 'nondefault', item.rna.volume != rnaDefaultVolume);
				var rnaConc = Doc.element('rnaConc'+i);
				rnaConc.innerHTML = Meludi.formatNumber(item.rna.ndConc) || '-';
				rnaConc.title = '260/280=' + item.rna.nd260by280 + '\n260/230=' + item.rna.nd260by230;
				
				item.rna.quantity = item.rna.ndConc * item.rna.volume / 1000;
				var enoughQuantity = item.rna.quantity >= MINIMAL_PRE_NORMALIZE_QUANTITY;
				item.rna.flag = enoughQuantity ? null : 'NotEnoughRemainingQuantity';

				Doc.element('rnaQuantity'+i).innerHTML = Meludi.formatNumber(item.rna.quantity || null, null, 2) || '-';
*/
				var cmnt = Doc.element('comments'+i);
				var tooltip = [];
				var comment = [];
				if (item.comment || item.dna.comment || item.rna.comment)
				{
					comment[0] = 'Yes';
					if (item.comment) tooltip[tooltip.length] = 'Specimen: ' + item.comment;
					if (item.rna.comment) tooltip[tooltip.length] = 'RNA: ' + item.rna.comment;
					if (item.dna.comment) tooltip[tooltip.length] = 'DNA: ' + item.dna.comment;
				}
				
				var modifiedReagents = false;
				for (var rgNo = 0; rgNo < exprep.REAGENTS.length; rgNo++)
				{
					var rg = exprep.REAGENTS[rgNo];
					if (item[rg]) 
					{
						tooltip[tooltip.length] = Doc.element(rg+'.title').innerHTML + ': ' + Strings.encodeTags(item[rg]);
						modifiedReagents = true;
					}
				}
				if (modifiedReagents)
				{
					comment[comment.length] = 'Reagents';
				}
				
				if (tooltip.length > 0)
				{
					cmnt.innerHTML = comment.join(', ');
					Doc.addClass(cmnt, 'nondefault');
					cmnt.title = tooltip.join('\n');
				}
				else
				{
					cmnt.innerHTML = '';
					Doc.removeClass(cmnt, 'nondefault');
					cmnt.title = null;
				}
			}
		}

		if (duplicatePosition)
		{
			var duplicatePositionStr = exprep.getQiacubePosition(duplicatePosition);
			Wizard.setInputStatus('downloadSampleIdFile', 'invalid', 'Duplicate QIAcube position: ' + duplicatePositionStr);
			qiacubePositionsAreValid = false;
		}
		else
		{
			Wizard.setInputStatus('downloadSampleIdFile');
			qiacubePositionsAreValid = true;
		}
	}

	/**
		Returns a pure integer position value from a QIAcube position
		string with optional instrument letter prefix A-Z.
		Returns `null` if input position string is `null`.

		positionStr = integer          => QIAcube position = integer (1-12)
		positionStr = Prefix + integer => QIAcube position = integer + 12*<Prefix order number 0-25>
	 */
	exprep.getIntQiacubePosition = function(positionStr)
	{
		var intPos = null;
		if (positionStr)
		{
			// Check if string contains uppercase letter + QIAcube position number 1-12
			if (/^[A-Z]([2-9]|1[0-2]?)$/.test(positionStr))
			{
				var qiaCubePrefix = positionStr.substring(0,1);
				// Get QIAcube prefix order number 0-25
				var qiacubePrefixOrder = qiacubeSortedPrefixes.indexOf(qiaCubePrefix);
				if (qiacubePrefixOrder >= 0)
				{
					intPos = parseInt(positionStr.substring(1)) + 12*qiacubePrefixOrder;
				}
			}
			if (intPos == null)
			{
				intPos = parseInt(positionStr);
			}
		}
		return intPos;
	}

	/**
		Returns a QIAcube position string with optional instrument
		letter prefix A-Z from a pure integer position value.

		Examples for qiacubeSortedPrefixes = 'CABDEFGHIJKLMNOPQRSTUVWXYZ' ("C" is default)
		intPosition (  1 -  12) => positionStr = 'C' + integer
		intPosition ( 13 -  24) => positionStr = 'A' + (integer - 12)
		intPosition (301 - 312) => positionStr = 'Z' + (integer - 300)
	 */
	exprep.getQiacubePosition = function(intPosition)
	{
		var qiacubePositionStr = null;
		if (intPosition && intPosition > 0)
		{
			var numPrefixes = qiacubeSortedPrefixes.length;
			var prefixIndex = Math.floor(intPosition/12);
			if (prefixIndex < numPrefixes)
			{
				var qiacubePrefix = qiacubeSortedPrefixes.substring(prefixIndex, prefixIndex + 1);
				var qiacubePos = intPosition - 12*prefixIndex;
				qiacubePositionStr = qiacubePrefix + qiacubePos;
			}
			else
			{
				qiacubePositionStr = '' + intPosition;
			}
		}
		return qiacubePositionStr;
	}

	exprep.editDnaRnaDetails = function(event)
	{
		var index = Data.get(event.currentTarget, 'index');
		var frm = document.forms['meludi'];
		// Get list of selected extract source items
		var selItemsList = exprep.getSelectedItemsList();
		var numItems = selItemsList.length;
/*
		var item = selItemsList[index];
*/
		var item = null;
		if (index < numItems)
		{
			item = selItemsList[index];
		}
		else
		{
			// Extra QIAcube item
			item = extraItemsList[index - numItems];
		}
		//var url = 'edit_dnarna_details.jsp?ID='+App.getSessionId();
		var url = 'edit_dnarna_prep_details.jsp?ID='+App.getSessionId();
		url += '&index='+index;
		url += '&mode=' + 'ExtractionPrep';
		//url += '&specimen=' + frm.extractSourceItems[index].specimen.name;
		url += '&specimen=' + item.name;
		Dialogs.openPopup(url, 'EditDnaRnaDetails'+index, 750, 500);
	}
	
	exprep.getNumber = function(sValue)
	{
		return parseFloat(sValue.replace(/,/, '.'));
	}

	exprep.downloadSampleIdFile = function()
	{
		var frm = document.forms['meludi'];

		if (!qiacubePositionsAreValid)
		{
			Forms.showNotification('downloadSampleIdFile', 'QIAcube positions are not valid. Please fix before downloading the file.');
			return;
		}

		var itemNames = [];
		var url = '../Extraction.servlet?ID='+App.getSessionId();
		url += '&cmd=DownloadNanoDropSampleIdFile';
		url += '&startPlateName='+encodeURIComponent(frm.startPlateName.value);
		url += '&qiacubeDate='+encodeURIComponent(frm.qiacubeDate.value);
		url += '&qiacubeRunNo='+encodeURIComponent(frm.qiacubeRunNo.value);
		
		// Get list of selected extract source items
		var selItemsList = exprep.getSelectedItemsList();
		var numItems = selItemsList.length;
		var itemNo = 0;
		for (var i = 0; i < numItems; i++)
		{
			var item = selItemsList[i];
			if (exprep.isSpecimen(item) || exprep.isRna(item))
			{
				itemNames[itemNo] = item.name;
				itemNo++;
			}
		}
		url += '&itemNames=' + encodeURIComponent(itemNames.join(','));
		window.open(url);
	}

	exprep.downloadQPcrSampleIdFile = function()
	{
		var frm = document.forms['meludi'];
		var itemNames = [];
		var url = '../Extraction.servlet?ID='+App.getSessionId();
		url += '&cmd=DownloadQPcrSampleIdFile';
		url += '&startPlateName='+encodeURIComponent(frm.startPlateName.value);
		url += '&qiacubeDate='+encodeURIComponent(frm.qiacubeDate.value);
		url += '&qiacubeRunNo='+encodeURIComponent(frm.qiacubeRunNo.value);

		// Get list of selected extract source items
		var selItemsList = exprep.getSelectedItemsList();
		var numItems = selItemsList.length;
		var itemNo = 0;
		for (var i = 0; i < numItems; i++)
		{
			var item = selItemsList[i];
			if (exprep.isSpecimen(item) || exprep.isDna(item))
			{
				itemNames[itemNo] = item.name;
				itemNo++;
			}
		}
		url += '&itemNames=' + encodeURIComponent(itemNames.join(','));
		window.open(url);
	}

	exprep.validateStep3 = function(event)
	{
		var valid = true;
		valid &= qiacubePositionsAreValid;
		
		if (!valid) event.preventDefault();
	}

	exprep.submit = function()
	{
		//alert("exprep.submit(): Start");
		var frm = document.forms['meludi'];
		
		var submitInfo = {};
		submitInfo.items = [];

		// Get list of selected extract source items
		var selItemsList = exprep.getSelectedItemsList();
		var numItems = selItemsList.length;
		// Get number of extra QIAcube items
		var numExtraQiacubeItems = 0;
		var numExtraQiacubeItemsStr = frm.numExtraQiacubeItems.value;
		if (numExtraQiacubeItemsStr && (numExtraQiacubeItemsStr != ''))
		{
			numExtraQiacubeItems = parseInt(numExtraQiacubeItemsStr, 10);
		}
		submitInfo.numExtraQiacubeItems = numExtraQiacubeItems;
		for (var i = 0; i < numItems; i++)
		{
			var item = selItemsList[i];
			submitInfo.items[submitInfo.items.length] = item;
		}
		
		// Extraction
		submitInfo.startPlateName = frm.startPlateName.value;
		submitInfo.isolationDate = frm.isolationDate.value;
		submitInfo.extractionProtocol = parseInt(frm.extractionProtocol.value, 10);
		var extractionProtocolName = '';
		var extractionProtocol = frm.extractionProtocol.value;
		if (extractionProtocol)
		{
			extractionProtocolName = protocolMap[extractionProtocol];
		}
		submitInfo.extractionProtocolName = extractionProtocolName;
/*
		submitInfo.rnaDefaultVolume = exprep.getNumber(frm.rnaDefaultVolume.value);
		submitInfo.dnaDefaultVolume = exprep.getNumber(frm.dnaDefaultVolume.value);
*/

		// QIAcube
		submitInfo.qiacubeDate = frm.qiacubeDate.value;
		submitInfo.qiacubePrimaryPrefix = frm.qiacubePrimaryPrefix.value;
		submitInfo.qiacubeSecondaryPrefix = frm.qiacubeSecondaryPrefix.value;
		submitInfo.qiacubeRunNo = parseInt(frm.qiacubeRunNo.value, 10);
		submitInfo.qiacubeOperator = frm.qiacubeOperator.value;
		submitInfo.allPrepFfpeKit = frm.allPrepFfpeKit.value;

/*
		// Reagents
		for (var i = 0; i < exprep.REAGENTS.length; i++)
		{
			var rg = exprep.REAGENTS[i];
			submitInfo[rg] = frm[rg].value;
		}
*/
		var url = '../Extraction.servlet?ID='+App.getSessionId();
		url += '&cmd=RegisterStartPlate';
		//alert("exprep.submit(): url = " + url);
		Wizard.showLoadingAnimation('Performing registration...');
		Wizard.asyncJsonRequest(url, exprep.submissionResults, 'POST', JSON.stringify(submitInfo));
	}
	
	exprep.submissionResults = function(response)
	{
		Wizard.showFinalMessage(response.messages);
		Doc.show('gorestart');
		Doc.show('gocreate');
		trackingSheetPreview = false;
		if (response.createdDnaPlate && response.createdDnaPlate.length > 0)
		{
			createdDnaPlate = response.createdDnaPlate;
		}
	}

	exprep.createProtocol = function()
	{
		var frm = document.forms['meludi'];
		var submitInfo = {};
		submitInfo.items = [];

		submitInfo.docMode = 'sheet';
		// Get list of selected extract source items
		var selItemsList = exprep.getSelectedItemsList();
		var numItems = selItemsList.length;
		// Get number of extra QIAcube items
		var numExtraQiacubeItems = 0;
		var numExtraQiacubeItemsStr = frm.numExtraQiacubeItems.value;
		if (numExtraQiacubeItemsStr && (numExtraQiacubeItemsStr != ''))
		{
			numExtraQiacubeItems = parseInt(numExtraQiacubeItemsStr, 10);
		}
		submitInfo.numExtraQiacubeItems = numExtraQiacubeItems;
		var totNumItems = numItems + numExtraQiacubeItems;
/*
		for (var i = 0; i < numItems; i++)
*/
		for (var i = 0; i < totNumItems; i++)
		{
/*
			var item = selItemsList[i];
*/
			var item = null;
			if (i < numItems)
			{
				item = selItemsList[i];
			}
			else
			{
				// Extra QIAcube item
				item = extraItemsList[i - numItems];
			}
			submitInfo.items[submitInfo.items.length] = item;
		}

		// Extraction
		submitInfo.startPlateName = frm.startPlateName.value;
		submitInfo.isolationDate = frm.isolationDate.value;
/*
		submitInfo.extractionProtocol = parseInt(frm.extractionProtocol.value, 10);
*/
		var extractionProtocolName = '';
		var extractionProtocol = frm.extractionProtocol.value;
		if (extractionProtocol)
		{
			extractionProtocolName = protocolMap[extractionProtocol];
		}
		var protocolVersionNo = '';
		if (extractionProtocolName)
		{
			protocolVersionNo = extractionProtocolName;
			// Remove part of string up to and including last "v"
			var vIndex = protocolVersionNo.indexOf('v');
			while (vIndex >= 0)
			{
				// Remove part of string up to and including first "v"
				protocolVersionNo = protocolVersionNo.substring(vIndex + 1);
				vIndex = protocolVersionNo.indexOf('v');
			}
		}
		var protocolPreviewFlag = null;
		if (trackingSheetPreview)
		{
			protocolPreviewFlag = 'true';
		}
		submitInfo.extractionProtocolName = extractionProtocolName;
		submitInfo.protocolVersionNo = protocolVersionNo;
		submitInfo.protocolPreviewFlag = protocolPreviewFlag;
/*
		submitInfo.dnaDefaultVolume = exprep.getNumber(frm.dnaDefaultVolume.value);
*/
		submitInfo.dnaDefaultVolume = null;
		
		// QIAcube
		submitInfo.qiacubeDate = frm.qiacubeDate.value;
		submitInfo.qiacubeRunNo = parseInt(frm.qiacubeRunNo.value, 10);
		submitInfo.qiacubeOperator = frm.qiacubeOperator.value;
		submitInfo.allPrepFfpeKit = frm.allPrepFfpeKit.value;

		// Save values in hidden input fields for retrieval after HTML request
		var submitInfoJsonStr = JSON.stringify(submitInfo);
		frm.hiddenSubmitInfo.value = submitInfoJsonStr;
		frm.hiddenNumItems.value = submitInfo.items.length;
		frm.hiddenCounterBalanceInfoRow.value = counterBalanceInfoRow;

		// Calling frm.submit() will change jsp file to extraction_protocol2.jsp
		frm.submit();
	}

	return exprep;
}();

Doc.onLoad(ExtractionPrep.initPage);

