var ExtractionProtocol = function()
{
	var ep = {};
	var debug = 0;
	
	ep.TITLE = 'MeLuDI';
	var COLORS = ['blue', 'green', 'yellow', 'red'];
	var nextColor = 0;
	var meludiVersion;

	// Page initialization
	ep.initPage = function()
	{
		Buttons.addClickHandler('print-button', Wizard.goPrint);
		var numItems = Data.get('page-data', 'numitems');
		var submitInfoJsonStr = Data.get('page-data', 'submitinfo');
		meludiVersion = Data.get('page-data', 'meludiversion');
		ep.initializeProtocol(submitInfoJsonStr);
	}

	ep.initializeProtocol = function(submitInfoJsonStr)
	{
		var frm = document.forms['meludi'];

		var submitInfo = JSON.parse(submitInfoJsonStr);
		var docMode = submitInfo.docMode;
		var itemInfo = submitInfo.items;
		var startPlateName = submitInfo.startPlateName;
		var qiacubeOperator = submitInfo.qiacubeOperator;
		var isolationDate = submitInfo.isolationDate;
		var allPrepFfpeKit = submitInfo.allPrepFfpeKit;
		var dnaDefaultVolume = submitInfo.dnaDefaultVolume;
		var dnaProtocolName = submitInfo.dnaProtocolName;
		var rnaProtocolName = submitInfo.rnaProtocolName;
		var protocolVersionNo = submitInfo.protocolVersionNo;
		var protocolPreviewFlag = submitInfo.protocolPreviewFlag;
		var numQiacubeItems = itemInfo.length;

		// Header data
		var docModeText = 'Sheet';
		if (docMode && docMode == 'report')
		{
			docModeText = 'Report';
		}
		if (protocolVersionNo == null || protocolVersionNo == '')
		{
			protocolVersionNo = '?';
		}
		var protocolPreviewText = '';
		if (protocolPreviewFlag != null && protocolPreviewFlag == 'true')
		{
			protocolPreviewText = ' <i>(preview)</i>';
		}
		var headerText = 'Lab Tracking ' + docModeText + ' for Allprep isolation v.' + protocolVersionNo + protocolPreviewText + ' <span class="meludi">' + ExtractionProtocol.TITLE + ' ' + meludiVersion + '</span>';
		Doc.element('allprepLabTrackingSheetHeader').innerHTML = headerText;
		Doc.element('startplateid').innerHTML = Strings.encodeTags(startPlateName);
		Doc.element('operator').innerHTML = Strings.encodeTags(qiacubeOperator);
		Doc.element('isolationdate').innerHTML = Strings.encodeTags(isolationDate);
		Doc.element('ffpeDnaRnaKitLotNo').innerHTML = Strings.encodeTags(allPrepFfpeKit);
		if (dnaDefaultVolume)
		{
			Doc.element('dnaElutionVolume').innerHTML = ep.encodeFloatTags(dnaDefaultVolume) + ' µl';
		}

		// Item data
		var accumItemNo = 0;
		accumItemNo = tableRowsForItems(accumItemNo, itemInfo, 'Item', 'item');
		accumItemNo = tableRowForOptionalCounterbalanceItem(accumItemNo, itemInfo, 'Item', 'item');

		Doc.show('all-protocol');
	}
	
	function tableRowsForItems(
		accumItemNo,
		itemJsonArr,
		itemType,
		itemTypeVarName)
	{
		// Table item row
		var html = '';
		if (itemJsonArr && itemJsonArr.length > 0)
		{
			for (var i = 0; i < itemJsonArr.length; i++)
			{
				var item = itemJsonArr[i];
/*
				var storageId = '-';
				var storagePosition = '-';
*/
				var qiacubePosition = '-';
				var platePos = '-';
				var rnaConc = null;
				var dnaConc = null;
				var dnaDeltaCt = null;
				var specimenComment = null;
				var rnaComment = null;
				var dnaComment = null;
				var comment = null;
				if (item)
				{
/*
					storageId = ep.asBioPlateName(item.bioWell);
					storagePosition = ep.asPureBioPlateLocation(item.bioWell);
*/
					qiacubePosition = item.qiacubePosition;
					var rna = item.rna;
					if (rna)
					{
						platePos = rna.well;
						rnaConc = rna.ndConc;
						rnaComment = rna.comment;
					}

					var dna = item.dna;
					if (dna)
					{
						dnaConc = dna.qubitConc;
						dnaDeltaCt = dna.deltaCt;
						dnaComment = dna.comment;
					}
					var specimenComment = item.comment;
				}
				if (specimenComment)
				{
					comment = 'Specimen: ' + specimenComment;
				}
				if (rnaComment)
				{
					if (comment)
					{
						comment += ' ';
					}
					else
					{
						comment = '';
					}
					comment += 'RNA: ' + rnaComment;
				}
				if (dnaComment)
				{
					if (comment)
					{
						comment += ' ';
					}
					else
					{
						comment = '';
					}
					comment += 'DNA: ' + dnaComment;
				}
				//alert("tableRowsForItems(): i = " + i + " itemType = " + itemType + " name = " + item.name + " qiacubePosition = " + qiacubePosition + " platePos = " + platePos + " rnaConc = " + rnaConc + " dnaConc = " + dnaConc + " comment = " + comment);
				// Create HTML code for table column with checkbox for marking destroyed items
				// var selectedCheckboxHtml = '<td><b><div class=\"colorbox\"></div>Selected </b></td>';
				// Insert item data in table columns
				Doc.element('sample.'+accumItemNo).innerHTML = Strings.encodeTags(item.name);
/*
				Doc.element('storageId.'+accumItemNo).innerHTML = Strings.encodeTags(storageId);
				Doc.element('storagePosition.'+accumItemNo).innerHTML = Strings.encodeTags(storagePosition);
*/
				Doc.element('qiacubePosition.'+accumItemNo).innerHTML = Strings.encodeTags(qiacubePosition);
				Doc.element('nanodropPlatePosition.'+accumItemNo).innerHTML = Strings.encodeTags(platePos);
				Doc.element('rnaConc.'+accumItemNo).innerHTML = ep.encodeFloatTags(rnaConc);
				Doc.element('dnaConc.'+accumItemNo).innerHTML = ep.encodeFloatTags(dnaConc);
				Doc.element('deltaCt.'+accumItemNo).innerHTML = ep.encodeFloatTags(dnaDeltaCt);
				Doc.element('remark.'+accumItemNo).innerHTML = Strings.encodeTags(comment);
				
				// Increment accumulated item number
				accumItemNo++;
			}
		}
		return accumItemNo;
	}

	function tableRowForOptionalCounterbalanceItem(
		accumItemNo,
		itemJsonArr,
		itemType,
		itemTypeVarName)
	{
		// Find number of QIAcube items
		var lastQiacubeItem;
		var numQiacubeItems = 0;
		if (itemJsonArr && itemJsonArr.length > 0)
		{
			for (var i = 0; i < itemJsonArr.length; i++)
			{
				var item = itemJsonArr[i];
				if (item)
				{
					var qiacubePosition = item.qiacubePosition;
					if (qiacubePosition != null && qiacubePosition != '' && qiacubePosition != '-')
					{
						lastQiacubeItem = item;
						numQiacubeItems++;
					}
				}
			}
		}
		if (numQiacubeItems == 1 || numQiacubeItems == 11 || numQiacubeItems == 23)
		{
			// Table item row
			var html = '';
			var name = 'COUNTERBALANCE';
			var qiacubePosition = '-';
			var comment = 'Reminder to use counterbalance';
			var lastQiacubePos = lastQiacubeItem.qiacubePosition;
			var lastQiacubePrefix = lastQiacubePos.substring(0,1);
			if (numQiacubeItems == 1)
			{
				qiacubePosition = lastQiacubePrefix + '7';
			}
			else if (numQiacubeItems == 11)
			{
				qiacubePosition = lastQiacubePrefix + '12';
			}
			else if (numQiacubeItems == 23)
			{
				qiacubePosition = lastQiacubePrefix + '12';
			}
			// Create HTML code for table column with checkbox for marking destroyed items
			// Insert item data in table columns
			Doc.element('sample.'+accumItemNo).innerHTML = Strings.encodeTags(name);
			Doc.element('qiacubePosition.'+accumItemNo).innerHTML = Strings.encodeTags(qiacubePosition);
			Doc.element('nanodropPlatePosition.'+accumItemNo).innerHTML = Strings.encodeTags('-');
			Doc.element('rnaConc.'+accumItemNo).innerHTML = Strings.encodeTags('-');
			Doc.element('dnaConc.'+accumItemNo).innerHTML = Strings.encodeTags('-');
			Doc.element('deltaCt.'+accumItemNo).innerHTML = Strings.encodeTags('-');
			Doc.element('remark.'+accumItemNo).innerHTML = Strings.encodeTags(comment);
				
			// Increment accumulated item number
			accumItemNo++;
		}
		// Table item row
		return accumItemNo;
	}

	ep.encodeFloatTags = function(value)
	{
		var encodedValue = value;
		if (!value)
		{
			encodedValue = Strings.encodeTags(value);
		}
		return encodedValue;
	}

	// A biowell as plate name
	ep.asBioPlateName = function(well)
	{
		if (!well) return '';
		var plate = well.bioPlate;
		if (!plate) return '';
		var text = plate.name;
		return text;
	}

	// A biowell as the location
	ep.asPureBioPlateLocation = function(well)
	{
		if (!well) return '';
		var plate = well.bioPlate;
		var text = well.location;
		if (plate.storage)
		{
			var storage = plate.storage;
			var tmp = [];
			if (storage.name) tmp[tmp.length] = storage.name;
			if (storage.section) tmp[tmp.length] = 'section: ' +storage.section;
			if (storage.tray) tmp[tmp.length] = 'tray: ' +storage.tray;
			if (storage.position) tmp[tmp.length] = 'position: ' +storage.position;
			text = text + ' ' + tmp.join('; ');
		}
		return text;
	}

	return ep;
}();

Doc.onLoad(ExtractionProtocol.initPage);

